@@ -10,14 +10,16 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+import sys
+import os
 import logging
 
 from botocore.hooks import HierarchicalEmitter
-from botocore.hooks import AliasedEventEmitter
 
 log = logging.getLogger('awscli.plugin')
 
 BUILTIN_PLUGINS = {'__builtin__': 'awscli.handlers'}
+CLI_LEGACY_PLUGIN_PATH = 'cli_legacy_plugin_path'
 
 
 def load_plugins(plugin_mapping, event_hooks=None, include_builtins=True):
@@ -40,20 +42,32 @@ def load_plugins(plugin_mapping, event_hooks=None, include_builtins=True):
     :return: An event emitter object.
 
     """
-    if include_builtins:
-        plugin_mapping.update(BUILTIN_PLUGINS)
-    modules = _import_plugins(plugin_mapping)
     if event_hooks is None:
-        event_hooks = AliasedEventEmitter()
-    for name, plugin in zip(plugin_mapping.keys(), modules):
-        log.debug("Initializing plugin %s: %s", name, plugin)
-        plugin.awscli_initialize(event_hooks)
+        event_hooks = HierarchicalEmitter()
+    if include_builtins:
+        _load_plugins(BUILTIN_PLUGINS, event_hooks)
+    plugin_path = plugin_mapping.pop(CLI_LEGACY_PLUGIN_PATH, None)
+    if plugin_path is not None:
+        _add_plugin_path_to_sys_path(plugin_path)
+        _load_plugins(plugin_mapping, event_hooks)
+    else:
+        log.debug(
+            "cli_legacy_plugin_path not defined in plugin section. Not "
+            "importing additional plugins."
+        )
     return event_hooks
 
 
-def _import_plugins(plugin_names):
+def _load_plugins(plugin_mapping, event_hooks):
+    modules = _import_plugins(plugin_mapping)
+    for name, plugin in zip(plugin_mapping.keys(), modules):
+        log.debug("Initializing plugin %s: %s", name, plugin)
+        plugin.awscli_initialize(event_hooks)
+
+
+def _import_plugins(plugin_mapping):
     plugins = []
-    for name, path in plugin_names.items():
+    for name, path in plugin_mapping.items():
         log.debug("Importing plugin %s: %s", name, path)
         if '.' not in path:
             plugins.append(__import__(path))
@@ -62,3 +76,10 @@ def _import_plugins(plugin_names):
             module = __import__(path, fromlist=[module])
             plugins.append(module)
     return plugins
+
+
+def _add_plugin_path_to_sys_path(plugin_path):
+    for dirname in plugin_path.split(os.pathsep):
+        log.debug("Adding additional path from cli_legacy_plugin_path "
+                  "configuration: %s", dirname)
+        sys.path.append(dirname)
